//////////////////////////////////////////////////////////////////////
// car.cpp: implementation for the Car class (Hierarchical Transformation).
// -Kyoung Shin Park (2013 Spring)
//////////////////////////////////////////////////////////////////////
#include "simpleCar2.h"

namespace moglclass
{
	
SimpleCar2::SimpleCar2()
{
	angle = 0.0f; // 0~360
	direction = 1.0f; // +/-1
	position = glm::vec3(0.0f, 0.0f, 0.0f);
	active = false;

	init();
}

void SimpleCar2::init()
{
	body = Parallelepiped(glm::vec3(0, 0, 0), glm::vec3(1, 0, 0), glm::vec3(0, 0, 1), glm::vec3(0, 1, 0));
	body.setColor(glm::vec3(0.0f, 1.0f, 0.0f));
	light = Torus(glm::vec3(0, 0, 0), 0.5, 0.1, 16, 16);
	light.setColor(glm::vec3(1.0f, 1.0f, 0.0f));
	wheel = Torus(glm::vec3(0, 0, 0), 0.5, 0.1, 16, 16);
	wheel.setColor(glm::vec3(0.0f, 0.0f, 0.0f));

	updateTransform();
}

void SimpleCar2::draw(Program* p, glm::mat4& projection, glm::mat4& view, glm::mat4& model)
{
	p->useProgram();
	p->setUniform("gProjection", projection);
	p->setUniform("gView", view);

	glm::mat4 carMatrix = model * bodyTransform * glm::scale(glm::mat4(1.0f), glm::vec3(4, 2, 4));
	p->setUniform("gModel", carMatrix);
	body.draw();

	carMatrix = model * bodyTransform * glm::translate(glm::mat4(1.0f), glm::vec3(1, 2, 1)) * glm::scale(glm::mat4(1.0f), glm::vec3(2, 1, 2));
	p->setUniform("gModel", carMatrix);
	body.draw();

	glm::mat4 lightMatrix = model * bodyTransform  * glm::translate(glm::mat4(1.0f), glm::vec3(4, 1, 1)) * glm::rotate(glm::mat4(1.0f), (float)(M_PI / 2), glm::vec3(0.0f, 1.0f, 0.0f)) * glm::scale(glm::mat4(1.0f), glm::vec3(0.5, 0.5, 1.5));
	p->setUniform("gModel", lightMatrix);
	light.draw();

	lightMatrix = model * bodyTransform  * glm::translate(glm::mat4(1.0f), glm::vec3(4, 1, 3)) * glm::rotate(glm::mat4(1.0f), (float)(M_PI / 2), glm::vec3(0.0f, 1.0f, 0.0f)) * glm::scale(glm::mat4(1.0f), glm::vec3(0.5, 0.5, 1.5));
	p->setUniform("gModel", lightMatrix);
	light.draw();

	glm::mat4 wheelMatrix = model * bodyTransform * glm::translate(glm::mat4(1.0f), glm::vec3(1, 0.0f, 0)) * wheelTransform;
	p->setUniform("gModel", wheelMatrix);
	wheel.draw();
	
	wheelMatrix = model * bodyTransform * glm::translate(glm::mat4(1.0f), glm::vec3(1, 0.0f, 4)) * wheelTransform;
	p->setUniform("gModel", wheelMatrix);
	wheel.draw();

	wheelMatrix = model * bodyTransform * glm::translate(glm::mat4(1.0f), glm::vec3(3, 0.0f, 0)) * wheelTransform;
	p->setUniform("gModel", wheelMatrix);
	wheel.draw();

	wheelMatrix = model * bodyTransform * glm::translate(glm::mat4(1.0f), glm::vec3(3, 0.0f, 4)) * wheelTransform;
	p->setUniform("gModel", wheelMatrix);
	wheel.draw();
	
}

void SimpleCar2::updateTransform()
{
	bodyTransform = glm::translate(glm::mat4(1.0f), position); // RHS x+ right
	wheelTransform = glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 0.0f, 1.0f));
}

bool SimpleCar2::update(float deltaTime)
{
	if (active)
	{
		angle = angle - 180.0f * (float)(deltaTime)* 0.0001f * direction;
		position[0] = position[0] + (float)(deltaTime)* 0.001f * direction;
		if (position[0] * direction > 3)
			direction = -direction;
		updateTransform();
	}
	return true;
}

} // moglclass namespace