//////////////////////////////////////////////////////////////////////
// cube3d.cpp: implementation for the Car class (Hierarchical Transformation).
// -Kyoung Shin Park (2016 Spring)
//////////////////////////////////////////////////////////////////////
#include "simpleCube3d.h"

namespace moglclass
{
	
SimpleCube3d::SimpleCube3d()
{
	angle = 0.0f; // 0~360
	active = false;
	init();
}

void SimpleCube3d::init()
{
	face[0] = Quad(glm::vec3(0, 0, 0), glm::vec3(0, 1, 0), glm::vec3(0, 0, -1), 2, 2);
	face[0].setColor(glm::vec3(1.0f, 1.0f, 0.0f)); // bottom yellow
	face[1] = Quad(glm::vec3(0, 0, 0), glm::vec3(0, 1, 0), glm::vec3(0, 0, -1), 2, 2);
	face[1].setColor(glm::vec3(0.0f, 1.0f, 0.0f)); // left green
	face[2] = Quad(glm::vec3(0, 0, 0), glm::vec3(0, 1, 0), glm::vec3(0, 0, -1), 2, 2);
	face[2].setColor(glm::vec3(0.0f, 1.0f, 1.0f)); // right cyan
	face[3] = Quad(glm::vec3(0, 0, 0), glm::vec3(0, 1, 0), glm::vec3(0, 0, -1), 2, 2);
	face[3].setColor(glm::vec3(1.0f, 0.0f, 0.0f)); // front red
	face[4] = Quad(glm::vec3(0, 0, 0), glm::vec3(0, 1, 0), glm::vec3(0, 0, -1), 2, 2);
	face[4].setColor(glm::vec3(0.0f, 0.0f, 1.0f)); // back blue
	face[5] = Quad(glm::vec3(0, 0, 0), glm::vec3(0, 1, 0), glm::vec3(0, 0, -1), 2, 2);
	face[5].setColor(glm::vec3(1.0f, 0.0f, 1.0f)); // top margenta

	updateTransform();
}

void SimpleCube3d::draw(Program* p, glm::mat4& projection, glm::mat4& view, glm::mat4& model)
{
	p->useProgram();
	p->setUniform("gProjection", projection);
	p->setUniform("gView", view);

	glm::mat4 m = model * faceTransform[0];
	p->setUniform("gModel", m);
	face[0].draw(); // bottom yellow

	m = model * faceTransform[0] * faceTransform[1];
	p->setUniform("gModel", m);
	face[1].draw(); // left green

	m = model * faceTransform[0] * faceTransform[2];
	p->setUniform("gModel", m);
	face[2].draw(); // right cyan

	m = model * faceTransform[0] * faceTransform[3];
	p->setUniform("gModel", m);
	face[3].draw(); // front red

	m = model * faceTransform[0] * faceTransform[4];
	p->setUniform("gModel", m);
	face[4].draw();

	m = model * faceTransform[0] * faceTransform[4] * faceTransform[4]; // top magenta (belongs to back face)
	p->setUniform("gModel", m);
	face[5].draw();
}

void SimpleCube3d::updateTransform()
{
	faceTransform[0] = glm::translate(glm::mat4(1.0f), glm::vec3(0, -1, 0)); 
	faceTransform[1] = glm::translate(glm::mat4(1.0f), glm::vec3(-1, 0, 0)) * glm::rotate(glm::mat4(1.0f), glm::radians(-angle), glm::vec3(0, 0, 1)) * glm::translate(glm::mat4(1.0f), glm::vec3(1, 0, 0)) * glm::translate(glm::mat4(1.0f), glm::vec3(-2, 0, 0));
	faceTransform[2] = glm::translate(glm::mat4(1.0f), glm::vec3(1, 0, 0)) * glm::rotate(glm::mat4(1.0f), glm::radians(angle), glm::vec3(0, 0, 1)) * glm::translate(glm::mat4(1.0f), glm::vec3(-1, 0, 0)) * glm::translate(glm::mat4(1.0f), glm::vec3(2, 0, 0));
	faceTransform[3] = glm::translate(glm::mat4(1.0f), glm::vec3(0, 0, 1)) * glm::rotate(glm::mat4(1.0f), glm::radians (-angle), glm::vec3(1, 0, 0)) * glm::translate(glm::mat4(1.0f), glm::vec3(0, 0, -1)) * glm::translate(glm::mat4(1.0f), glm::vec3(0, 0, 2));
	faceTransform[4] = glm::translate(glm::mat4(1.0f), glm::vec3(0, 0, -1)) * glm::rotate(glm::mat4(1.0f), glm::radians(angle), glm::vec3(1, 0, 0)) * glm::translate(glm::mat4(1.0f), glm::vec3(0, 0, 1)) * glm::translate(glm::mat4(1.0f), glm::vec3(0, 0, -2));
}

bool SimpleCube3d::update(float deltaTime)
{
	if (active)
	{
		angle = (float)(deltaTime) * 0.01f;
		if (angle > 90) angle = 90;
		updateTransform();
	}

	return true;
}

} // moglclass namespace