//////////////////////////////////////////////////////////////////////
// mobile.cpp: implementation for the Mobile class (Hierarchical Transformation).
// -Kyoung Shin Park (2013 Spring)
//////////////////////////////////////////////////////////////////////
#include "simpleMobile.h"

namespace moglclass
{
	
SimpleMobile::SimpleMobile()
{
	angle = 0.0f; // 0~360
	direction = 1.0f; // +/-1
	position = glm::vec3(-2.0f, 0.0f, 0.0f);

	active = false;

	init();
}

SimpleMobile::~SimpleMobile()
{
	for (int i=0; i<8; i++) delete mobile[i];
	std::cout << "~SimpleMobile" << std::endl;
}

void SimpleMobile::init()
{
	mobile[0] = new Parallelepiped(glm::vec3(0.0f, 5.0f, 0.0f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 1.0f), glm::vec3(0.0f, 3.0f, 0.0f));
	mobile[0]->setColor(glm::vec3(1,0,0));
	mobile[1] = new Parallelepiped(glm::vec3(-4.0f, 0.0f, 4.0f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 1.0f), glm::vec3(4.0f, 5.0f, -4.0f));
	mobile[1]->setColor(glm::vec3(0,1,0));
	mobile[2] = new Parallelepiped(glm::vec3(4.0f, 0.0f, 4.0f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 1.0f), glm::vec3(-4.0f, 5.0f, -4.0f));
	mobile[2]->setColor(glm::vec3(0,0,1));
	mobile[3] = new Parallelepiped(glm::vec3(0.0f, 0.0f, -4.0f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 1.0f), glm::vec3(0.0f, 5.0f, 4.0f));
	mobile[3]->setColor(glm::vec3(1,1,0));

	mobile[4] = new Cube(glm::vec3(0, 0, 0), 7.0f);
	mobile[4]->setColor(glm::vec3(1,1,0));
	mobile[5] = new Torus(glm::vec3(0, 0, 0), 3.0f, 1.0f, 32, 16);
	mobile[5]->setColor(glm::vec3(0,1,1));
	mobile[6] = new Sphere(glm::vec3(0, 0, 0), 3.5f, 16, 16);
	mobile[6]->setColor(glm::vec3(1,0,1));
	mobile[7] = new Cylinder(glm::vec3(0, 0, 0), 3.5f, 4.0f, 4);
	mobile[7]->setColor(glm::vec3(0.7,0.5,1));
}

void SimpleMobile::draw(Program* p, glm::mat4& projection, glm::mat4& view, glm::mat4& model)
{
	p->useProgram();
	p->setUniform("gProjection", projection);
	p->setUniform("gView", view);

	glm::mat4 mainMatrix = model * bodyTransform;
	p->setUniform("gModel", mainMatrix);
	for (int i=0; i<4; i++) mobile[i]->draw();

	glm::mat4 mobileMatrix = model * bodyTransform * wheelTransform[0];
	p->setUniform("gModel", mobileMatrix);
	mobile[4]->draw();

	mobileMatrix = model * bodyTransform * wheelTransform[1];
	p->setUniform("gModel", mobileMatrix);
	mobile[5]->draw();

	mobileMatrix = model * bodyTransform * wheelTransform[2];
	p->setUniform("gModel", mobileMatrix);
	mobile[6]->draw();

	mobileMatrix = model * bodyTransform * wheelTransform[2] * wheelTransform[3];
	p->setUniform("gModel", mobileMatrix);
	mobile[7]->draw();
}

bool SimpleMobile::update(float deltaTime)
{
	if (active)
	{
		angle = angle - 180.0f * (float) (deltaTime) * 0.0001f;
	}

	bodyTransform = glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0, 1, 0));
	wheelTransform[0] = glm::translate(glm::mat4(1.0f), glm::vec3(-4.0f, -3.0f, 4.0f)) * glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 0.0f, 1.0f)); // cube
	wheelTransform[1] = glm::translate(glm::mat4(1.0f), glm::vec3(4.0f, -3.0f, 4.0f)) * glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 1.0f, 0.0f)); // torus
	wheelTransform[2] = glm::translate(glm::mat4(1.0f), glm::vec3(0.0f, -3.0f, -4.0f)) * glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 1.0f, 0.0f)); // sphere
	wheelTransform[3] = glm::translate(glm::mat4(1.0f), glm::vec3(0.0f, -5.0f, 0.0f)) * glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 1.0f, 0.0f)); // cylinder

	return true;
}

} // moglclass namespace