//////////////////////////////////////////////////////////////////////
// simpleSolar.cpp: implementation for the Simple Solar System class (Hierarchical Transformation).
// -Kyoung Shin Park (2013 Spring)
//////////////////////////////////////////////////////////////////////
#include "simpleSolar.h"

namespace moglclass
{

const float SimpleSolar::SunRadius = 4.0f;
const float SimpleSolar::EarthRadius = 1.0f;
const float SimpleSolar::MoonRadius = 0.5f;
const float SimpleSolar::EarthDistanceFromSun = 10.0f;
const float SimpleSolar::MoonDistanceFromEarth = 2.0f;

SimpleSolar::SimpleSolar()
{
	sunSpin = 0.0f;// sun
	earthSpin = 0.0f;// earth spin
	earthOrbit = 0.0f;// earth orbit around the sun
	moonSpin = 0.0f;// moon spin
	moonOrbit = 0.0f;// moon orbit around the earth

	active = false;

	init();
}

SimpleSolar::~SimpleSolar()
{ 
	if (sun) delete sun;
	if (earth) delete earth;
	if (moon) delete moon;
	std::cout << "~SimpleSolar" << std::endl; 
}

void SimpleSolar::init()
{
	sun = new Sphere(glm::vec3(0, 0, 0), SunRadius, 16, 16);		
	sun->setColor(glm::vec3(1,1,0)); // yellow
	earth = new Sphere(glm::vec3(0, 0, 0), EarthRadius, 16, 16);
	earth->setColor(glm::vec3(0,1,0)); // green
	moon = new Sphere(glm::vec3(0, 0, 0), MoonRadius, 16, 16);
	moon->setColor(glm::vec3(0.5f, 0.5f, 0.5f)); // gray
}

void SimpleSolar::draw(Program* p, glm::mat4& projection, glm::mat4& view, glm::mat4& model)
{
	p->useProgram();
	p->setUniform("gProjection", projection);
	p->setUniform("gView", view);

	glm::mat4 sunMatrix = model * sunTransform;
	p->setUniform("gModel", sunMatrix);
	sun->draw();

	glm::mat4 earthMatrix = model * earthTransform;
	p->setUniform("gModel", earthMatrix);
	earth->draw();

	glm::mat4 moonMatrix = model * moonTransform;
	p->setUniform("gModel", moonMatrix);
	moon->draw();
}

bool SimpleSolar::update(float deltaTime)
{
	if (active)
	{
	// ¾ = ¾
	sunSpin += (float) (deltaTime) * 0.01f;
	sunTransform = glm::rotate(glm::mat4(1.0f), sunSpin, glm::vec3(0.0f, 1.0f, 0.0f));

	//  =  * 
	earthSpin += (float) (deltaTime) * 0.05f;
	earthOrbit += (float) (deltaTime) * 0.01f;
	earthTransform = glm::rotate(glm::mat4(1.0f), earthOrbit, glm::vec3(0.0f, 1.0f, 0.0f))
		* glm::translate(glm::mat4(1.0f), glm::vec3(0.0f, 0.0f, EarthDistanceFromSun))
		* glm::rotate(glm::mat4(1.0f), earthSpin, glm::vec3(0.0f, 1.0f, 0.0f)); 

	//  =  * ް * 
	moonSpin += (float) (deltaTime) * 0.07f;
	moonOrbit += (float) (deltaTime) * 0.08f;
	moonTransform = glm::rotate(glm::mat4(1.0f), earthOrbit, glm::vec3(0.0f, 1.0f, 0.0f)) 
		* glm::translate(glm::mat4(1.0f), glm::vec3(0.0f, 0.0f, EarthDistanceFromSun))
		* glm::rotate(glm::mat4(1.0f), moonOrbit, glm::vec3(0.0f, 1.0f, 0.0f)) 
		* glm::translate(glm::mat4(1.0f), glm::vec3(0.0f, 0.0f, MoonDistanceFromEarth))
		* glm::rotate(glm::mat4(1.0f), moonSpin, glm::vec3(0.0f, 1.0f, 0.0f)); 
	}

	return true;
}

} // moglclass namespace